"""Modify connection strategy workflow. Flipping the connection between in-band to out-of-band."""

from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, UUIDstr
from orchestrator.workflow import StepList, begin, done, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form
from pydantic import ConfigDict

from gso.products.product_types.router import Router
from gso.utils.shared_enums import ConnectionStrategy


def initial_input_form_generator(subscription_id: UUIDstr) -> FormGenerator:
    """Modify the connection strategy initial form."""
    subscription = Router.from_subscription(subscription_id)

    current_connection_strategy = (
        ConnectionStrategy.OUT_OF_BAND if subscription.router.router_access_via_ts else ConnectionStrategy.IN_BAND
    )

    class ModifyConnectionStrategyForm(FormPage):
        model_config = ConfigDict(title=f"Modify the connection strategy of {subscription.router.router_fqdn}.")

        connection_strategy: ConnectionStrategy = current_connection_strategy

    user_input = yield ModifyConnectionStrategyForm

    return user_input.model_dump()


@step("Update subscription model")
def update_subscription_model(subscription: Router, connection_strategy: str) -> State:
    """Update the database model to reflect the new connection strategy.

    If the connection strategy is set to IN-BAND, then access_via_ts should be set to False.
    Conversely, if the connection strategy is set to OUT-OF-BAND, access_via_ts should be set to True.
    """
    subscription.router.router_access_via_ts = connection_strategy == ConnectionStrategy.OUT_OF_BAND

    return {"subscription": subscription}


@workflow(
    "Modify connection strategy",
    initial_input_form=wrap_modify_initial_input_form(initial_input_form_generator),
    target=Target.MODIFY,
)
def modify_connection_strategy() -> StepList:
    """Modify the connection strategy."""
    return begin >> store_process_subscription(Target.MODIFY) >> unsync >> update_subscription_model >> resync >> done
